﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using System.Web.Http;
using HeuristicLab.Services.Hive;
using HeuristicLab.Services.Hive.DataAccess.Interfaces;
using DA = HeuristicLab.Services.Hive.DataAccess;
using DT = HeuristicLab.Services.WebApp.Statistics.WebApi.DataTransfer;

namespace HeuristicLab.Services.WebApp.Statistics.WebApi {
  public class GroupController : ApiController {
    private IPersistenceManager PersistenceManager {
      get { return ServiceLocator.Instance.PersistenceManager; }
    }

    public DT.GroupDetails GetGroupDetails(Guid id) {
      using (var pm = PersistenceManager) {
        var dimClientDao = pm.DimClientDao;
        var factClientInfoDao = pm.FactClientInfoDao;
        return pm.UseTransaction(() => {
          return (from client in dimClientDao.GetActiveClients().Where(x => x.ResourceGroupId == id)
                  join info in factClientInfoDao.GetAll()
                    on client.Id equals info.ClientId into clientInfoJoin
                  from clientInfo in clientInfoJoin.OrderByDescending(x => x.Time).Take(1)
                  let offline = (clientInfo.SlaveState == DA.SlaveState.Offline)
                  select new {
                    ResourceGroupId = client.ResourceGroupId,
                    GroupName = client.GroupName,
                    TotalCores = clientInfo.NumTotalCores,
                    UsedCores = offline ? 0 : clientInfo.NumUsedCores,
                    TotalMemory = clientInfo.TotalMemory,
                    UsedMemory = offline ? 0 : clientInfo.UsedMemory,
                    CpuUtilization = offline ? 0 : clientInfo.CpuUtilization
                  })
            .GroupBy(x => new { x.ResourceGroupId, x.GroupName })
            .Select(x => new DT.GroupDetails {
              Id = x.Key.ResourceGroupId ?? default(Guid),
              Name = x.Key.GroupName,
              Clients = x.Count(),
              TotalCores = x.Sum(y => y.TotalCores),
              UsedCores = x.Sum(y => y.UsedCores),
              TotalMemory = x.Sum(y => y.TotalMemory),
              UsedMemory = x.Sum(y => y.UsedMemory),
              CpuUtilization = x.Average(y => y.CpuUtilization)
            }).FirstOrDefault();
        });
      }
    }

    public DT.GroupPage GetGroups(int page, int size) {
      using (var pm = PersistenceManager) {
        var dimClientDao = pm.DimClientDao;
        var factClientInfoDao = pm.FactClientInfoDao;
        var data = (from client in dimClientDao.GetActiveClients()
                    join info in factClientInfoDao.GetAll()
                      on client.Id equals info.ClientId into clientInfoJoin
                    from clientInfo in clientInfoJoin.OrderByDescending(x => x.Time).Take(1)
                    let offline = (clientInfo.SlaveState == DA.SlaveState.Offline)
                    select new {
                      ResourceGroupId = client.ResourceGroupId,
                      TotalCores = clientInfo.NumTotalCores,
                      UsedCores = offline ? 0 : clientInfo.NumUsedCores,
                      TotalMemory = clientInfo.TotalMemory,
                      UsedMemory = offline ? 0 : clientInfo.UsedMemory,
                      CpuUtilization = offline ? 0 : clientInfo.CpuUtilization
                    }).GroupBy(x => x.ResourceGroupId).Select(x => new {
                      GroupId = x.Key,
                      Count = x.Count(),
                      TotalCores = x.Sum(y => y.TotalCores),
                      UsedCores = x.Sum(y => y.UsedCores),
                      TotalMemory = x.Sum(y => y.TotalMemory),
                      UsedMemory = x.Sum(y => y.UsedMemory),
                      CpuUtilization = x.Average(y => y.CpuUtilization)
                    });
        var query = dimClientDao.GetAll()
          .GroupBy(x => new { x.ResourceGroupId, x.GroupName })
          .Select(x => new {
            Id = x.Key.ResourceGroupId ?? default(Guid),
            Name = x.Key.GroupName
          });
        return pm.UseTransaction(() => new DT.GroupPage {
          TotalGroups = query.Count(),
          Groups = query
          .Skip((page - 1) * size)
          .Take(size)
          .Join(data, x => x.Id, y => y.GroupId,
            (group, info) => new DT.Group {
              Id = group.Id,
              Name = group.Name,
              Clients = info.Count,
              TotalCores = info.TotalCores,
              UsedCores = info.UsedCores,
              TotalMemory = info.TotalMemory,
              UsedMemory = info.UsedMemory,
              CpuUtilization = info.CpuUtilization
            }).ToList()
        });
      }
    }
  }
}
